{-# LANGUAGE NamedFieldPuns        #-}

module Schpin.Walker.Config where

import   Schpin.Shared
import           Schpin.Walker
import           Schpin.Package
import           Data.YAML
import           Data.ByteString.Lazy.UTF8      ( toString )
import           Data.Text                      ( pack )
import           Data.Maybe                     ( catMaybes )
import           Data.Scientific
import           Numeric.Units.Dimensional.Prelude
                                                ( (/~)
                                                , radian
                                                )

acTypeToString :: ActuatorType -> String
acTypeToString t = case t of
  Lewan     -> "LEWAN"
  LewanTest -> "LEWAN_TEST"
  Dynamixel -> "DYNMX"

writeControlConfig :: ExtendedWalker -> PackageConfig -> IO ()
writeControlConfig walker cfg = do
  makeParentDirectories filename
  writeFile filename $ toString $ encode1 out
 where
  filename :: FilePath
  filename = pkg_dir cfg ++ "/config/control.yml"

  out :: Node ()
  out =
    mapping [pack "control_node" .= mapping [pack "ros__parameters" .= params]]

  freq :: Double
  freq = 1.0

  params :: Node ()
  params = mapping
    [ pack "joint_state" .= pack "/control/joint_states"
    , pack "diag_freq" .= freq
    , pack "diag_topic" .= pack "/diagnostics"
    , pack "servos" .= servos
    ]

  servos :: Node ()
  servos =
    mapping
      $   catMaybes
      $   concat
      $   (\Leg { root_joint } -> mapJoints joint_to_conf root_joint)
      <$> (legs walker)

  joint_to_conf :: ExtendedJoint -> Maybe Pair
  joint_to_conf (ExtendedJoint (Joint { joint_name, jtype }) uname) = do
    conf <- cfg jtype
    return ((pack uname) .= conf)
   where

    cfg :: JointType -> Maybe (Node ())
    cfg RevoluteJoint { actuator, min_angle, max_angle } = Just $ mapping
      [ pack "id" .= ac_id actuator
      , pack "type" .= pack (acTypeToString $ ac_type actuator)
      , pack "joint_name" .= pack uname
      , pack "from_servo" .= ac_from actuator
      , pack "to_servo" .= ac_to actuator
      , pack "from_angle" .= (min_angle /~ radian)
      , pack "to_angle" .= (max_angle /~ radian)
      , pack "stall_range" .= (5 :: Int)
      , pack "angle_change" .= (5 :: Int)
      , pack "goto_p" .= ( 10.0 :: Double)
      , pack "goto_i" .= ( 0.0 :: Double)
      , pack "goto_d" .= ( 150.0 :: Double)
      , pack "vel_p" .= ( 0.6 :: Double)
      , pack "vel_i" .= ( 0.005 :: Double)
      , pack "vel_d" .= ( 0.6 :: Double)
      ]
    cfg FixedJoint = Nothing

